@@ -43,6 +43,31 @@ namespace xt
     auto broadcast(E&& e, const I (&s)[L]);
 #endif
 
+    /*************************
+     * xbroadcast extensions *
+     *************************/
+
+    namespace extension
+    {
+        template <class Tag, class CT, class X>
+        struct xbroadcast_base_impl;
+
+        template <class CT, class X>
+        struct xbroadcast_base_impl<xtensor_expression_tag, CT, X>
+        {
+            using type = xtensor_empty_base;
+        };
+
+        template <class CT, class X>
+        struct xbroadcast_base
+            : xbroadcast_base_impl<xexpression_tag_t<CT>, CT, X>
+        {
+        };
+
+        template <class CT, class X>
+        using xbroadcast_base_t = typename xbroadcast_base<CT, X>::type;
+    }
+
     /**************
      * xbroadcast *
      **************/
@@ -74,12 +99,15 @@ namespace xt
      */
     template <class CT, class X>
     class xbroadcast : public xexpression<xbroadcast<CT, X>>,
-                       public xconst_iterable<xbroadcast<CT, X>>
+                       public xconst_iterable<xbroadcast<CT, X>>,
+                       public extension::xbroadcast_base_t<CT, X>
     {
     public:
 
         using self_type = xbroadcast<CT, X>;
         using xexpression_type = std::decay_t<CT>;
+        using extension_base = extension::xbroadcast_base_t<CT, X>;
+        using expression_tag = typename extension_base::expression_tag;
 
         using value_type = typename xexpression_type::value_type;
         using reference = typename xexpression_type::reference;
@@ -128,11 +156,13 @@ namespace xt
         template <class It>
         const_reference element(It first, It last) const;
 
+        const xexpression_type& expression() const noexcept;
+
         template <class S>
         bool broadcast_shape(S& shape, bool reuse_cache = false) const;
 
         template <class S>
-        bool is_trivial_broadcast(const S& strides) const noexcept;
+        bool has_linear_assign(const S& strides) const noexcept;
 
         template <class S>
         const_stepper stepper_begin(const S& shape) const noexcept;
@@ -142,6 +172,12 @@ namespace xt
         template <class E, class XCT = CT, class = std::enable_if_t<xt::is_xscalar<XCT>::value>>
         void assign_to(xexpression<E>& e) const;
 
+        template <class E>
+        using rebind_t = xbroadcast<E, X>;
+
+        template <class E>
+        rebind_t<E> build_broadcast(E&& e) const;
+
     private:
 
         CT m_e;
@@ -371,6 +407,15 @@ namespace xt
     {
         return m_e.element(last - dimension(), last);
     }
+
+    /**
+     * Returns a constant reference to the underlying expression of the broadcast expression.
+     */
+    template <class CT, class X>
+    inline auto xbroadcast<CT, X>::expression() const noexcept -> const xexpression_type&
+    {
+        return m_e;
+    }
     //@}
 
     /**
@@ -391,17 +436,17 @@ namespace xt
     }
 
     /**
-     * Compares the specified strides with those of the container to see whether
-     * the broadcasting is trivial.
-     * @return a boolean indicating whether the broadcasting is trivial
+     * Checks whether the xbroadcast can be linearly assigned to an expression
+     * with the specified strides.
+     * @return a boolean indicating whether a linear assign is possible
      */
     template <class CT, class X>
     template <class S>
-    inline bool xbroadcast<CT, X>::is_trivial_broadcast(const S& strides) const noexcept
+    inline bool xbroadcast<CT, X>::has_linear_assign(const S& strides) const noexcept
     {
         return dimension() == m_e.dimension() &&
             std::equal(m_shape.cbegin(), m_shape.cend(), m_e.shape().cbegin()) &&
-            m_e.is_trivial_broadcast(strides);
+            m_e.has_linear_assign(strides);
     }
     //@}
 
@@ -429,6 +474,13 @@ namespace xt
         ed.resize(m_shape);
         std::fill(ed.begin(), ed.end(), m_e());
     }
+
+    template <class CT, class X>
+    template <class E>
+    inline auto xbroadcast<CT, X>::build_broadcast(E&& e) const -> rebind_t<E>
+    {
+        return rebind_t<E>(std::forward<E>(e), inner_shape_type(m_shape));
+    }
 }
 
 #endif
